//Libraries
#include <SPI.h>
#include <MFRC522.h>
#include <AccelStepper.h>

#define RST_PIN A5 //reset Pin RFID           
#define SS_PIN 10 //SDA PIN RFID
    
#define IND1 2    //First Inductive Sensor
#define IND2 4    //Second Inductive Sensor
#define COIN 7    //Coin Verify Unit
#define END 3     //final position Switch
#define RELAI 8  //Relai

#define dirPin A1    //Stepper direction Pin
#define stepPin 6   //Stepper step Pin
#define motorInterfaceType 1  //Stepper Interface Type
#define sleep A2    //Stepper sleep Pin

#define MOTOR 5     //Motor Pin
#define dirPIN A3   //Motor direction Pin

//=========================================== Sensor input declarations

 bool COINmem=0;    //Coin Verify Unit Memory
 bool COINval=0;    //Coin Verify Unit Value
 bool IND1mem=0;    //First Inductive Sensor Memory
 bool IND1val = 0;  //First Inductive Sensor Value
 bool IND2mem=0;    //Second Inductive Sensor Memory
 bool IND2val = 0;  //Second Inductive Sensor Value
 bool ENDmem=0;     //Final position Memory
 bool ENDval=0;     //Final position Value
 bool IND1reset=0;  //First Inductive Sensor Reset

 //=========================================== Raspberry Pi data declaration
 
 double design=0;                     //selected Design
 long unsigned int offset=0;          //selected Offset
 long unsigned int MOTORtimeout = 0;  //selected Motortimeout
 int MOTORspeed = 0;                  //Motor speed
 int STEPPERspeed = 0;                //Stepper speed

//=========================================== Counter and time dely declaration
 
 double INDcounter=0;
 int steps=0;
 long unsigned int previousMillis = 0;
 long unsigned int currentMillis = 0;
 long unsigned int previousMillis_timeout = 0;
 long unsigned int currentMillis_timeout = 0;

//=========================================== RFID declarations
 
 byte sector         = 1;
 byte blockAddr      = 4;
 byte trailerBlock   = 7;
 byte buffer[18];
 byte size = sizeof(buffer);
 MFRC522::StatusCode status;
 MFRC522 mfrc522(SS_PIN, RST_PIN);
 MFRC522::MIFARE_Key key;

//=========================================== RFID output declaration

 String code = "";
 String addzero = "";
 String pass = "";

//=========================================== Serial declarations
 
 String data = "";
 String subAblauf = "";
 String subLoop = "";
 String subLoop2 = "";

 AccelStepper stepper = AccelStepper(motorInterfaceType, stepPin, dirPin);

//=========================================== Setup
 
void setup() {

 pinMode(RELAI, OUTPUT);
 pinMode(IND1, INPUT_PULLUP); 
 pinMode(IND2, INPUT_PULLUP);
 pinMode(COIN, INPUT_PULLUP);
 pinMode(END, INPUT_PULLUP);
 pinMode(MOTOR, OUTPUT);

 digitalWrite(RELAI, LOW);

 pinMode(dirPIN, OUTPUT);
 stepper.setMaxSpeed(1000);
 stepper.setAcceleration(200);


 
 Serial.begin(9600);
 SPI.begin();
 for (byte i = 0; i < 6; i++) {
        key.keyByte[i] = 0xFF;
    }
    dump_byte_array(key.keyByte, MFRC522::MF_KEY_SIZE);
    Serial.println();
      
 RFID();
 ref();
}

//============================================================ Initialize RFID

void RFID() {
  
  mfrc522.PCD_Init();
}

//============================================================ Loop

void loop() 
{

//============================================================ Sensor one read

  IND1val = digitalRead(IND1); 
  if (!IND1val & !IND1mem) 
    { 
      /*if(INDcounter == 12 || IND1reset)
        {
      INDcounter = 0;
      IND1reset = 0;
        }*/
      IND1mem = 1;
      INDcounter += 1;
      Serial.println("IND1-" + String(INDcounter));
    }
  if(IND1val)
    {
      IND1mem = 0;
    }

  

//============================================================ Sensor two read
 
  IND2val = digitalRead(IND2); 
  if (!IND2val & !IND2mem) 
    {    
      Serial.println("IND2"); 
      IND1reset = 1;
      IND2mem = 1;
      INDcounter = 0;    /* SCAW */
    }
  if(IND2val)
    {
      IND2mem = 0;
      IND1reset = 0;
    }

//============================================================ Final position read

  ENDval = digitalRead(END); 
  if (ENDval & !ENDmem) 
    { 
      Serial.println("END1"); 
      ENDmem = 1;
      stepper.setCurrentPosition(0);
    }   
  if(!ENDval & ENDmem)
    {
      ENDmem=0;
      delay(1);
    }

//============================================================ Serial get data

  serial();
  subLoop = data.substring(0, 2);
  if (subLoop == "TO")
    {
      MOTORtimeout = (data.substring(3)).toInt();
      data ="";
    } 

  else if (subLoop == "SM")
    {
      MOTORspeed = (data.substring(3)).toInt();
      data ="";
    }

  else if (subLoop == "SS")
    {
      STEPPERspeed = (data.substring(3)).toInt();
      data ="";
    }
    
  else if (data == "Motor-right")
    {
      digitalWrite(dirPIN, HIGH);
      analogWrite(MOTOR, MOTORspeed);
      data = "";
    }
    
  else if (data == "Motor-left")
    {
      digitalWrite(dirPIN, LOW);
      analogWrite(MOTOR, MOTORspeed);
      data = "";
    }
    
  else if (data == "Motor-stop")
    {
      analogWrite(MOTOR, 0);
      data = "";
    }

//=========================================== Check Motortimeout
   
  // currentMillis_timeout = millis();
    
   //if (currentMillis_timeout - previousMillis_timeout >= (MOTORtimeout * 60 * 1000)) 
    //{
      //analogWrite(MOTOR, 0);
    //}

//============================================================ Coin Very Unit read
   
  COINval = digitalRead(COIN);
  if(COINval and steps==0)
    {
      Serial.println("Coin inserted");
      steps=10;
    }

//=========================================== flow control

  switch(steps)
    {
      case 10:
        Serial.println("choose design");
        Serial.println(MOTORspeed);
        analogWrite(MOTOR, MOTORspeed);
        digitalWrite(dirPIN, HIGH);
        digitalWrite(RELAI, HIGH);
        steps=20;
      break;
      
      case 20:  
        serial();
        subAblauf = data.substring(0, 4);
        if (subAblauf == "Data") 
          {
            design = (data.substring(5, 6)).toInt();
            offset = (data.substring(7)).toInt();
            Serial.println("design " + String(design) +" chosen | offset: "+ String(offset));
            steps=30; 
            data = "";
          } 
      break;

      case 30:
        if(INDcounter/2 == design)
          {
            previousMillis = millis();
            steps=40;
          }
      break;

      case 40:
        currentMillis = millis();
        if (currentMillis - previousMillis >= offset) 
          {
            steps = 50;
          }
      break;
      
      case 50:
        digitalWrite(sleep, LOW);
        while (stepper.currentPosition() != -144) 
          {
            stepper.setSpeed(-STEPPERspeed);
            stepper.runSpeed();
          }
        previousMillis = millis();
        steps = 60;
      break;   
         
      case 60: 
        currentMillis = millis();
        if (currentMillis - previousMillis >= 5000) 
          {
            previousMillis = millis();
            steps = 70;
          }
      break;
      
      case 70: 
        while (!ENDval) 
          {
            ENDval = digitalRead(END);
            stepper.setSpeed(STEPPERspeed);
            stepper.runSpeed();
            
              if (stepper.currentPosition() == 0) 
                {
                  digitalWrite(sleep, HIGH);
                   digitalWrite(RELAI, LOW);
                  Serial.println("Stepper fertig");
                  steps = 80;
                  delay(1000);
                  break;
                }  
          

        if (ENDval) 
          {
            Serial.println("Stepper fertig");
            digitalWrite(sleep, HIGH);
             digitalWrite(RELAI, LOW);
             delay(1000);
            stepper.setCurrentPosition(0);
            steps=80;   
          }
          }
      break;

      
      case 80: 
        previousMillis = millis();
        steps = 90;
      break;

      case 90:
        currentMillis = millis();
        if (currentMillis - previousMillis >= 5000) 
          {
            steps = 100;
          }
      break;

      case 100: 
        previousMillis_timeout = millis();
        analogWrite(MOTOR, 0);
        steps = 0;
      break;

      default:
      break;
    }

//=========================================== Authenticate RFID

  if ( ! mfrc522.PICC_IsNewCardPresent())
    return;

  if ( ! mfrc522.PICC_ReadCardSerial())
    return;
  
  status = ((MFRC522::StatusCode) mfrc522.PCD_Authenticate
  (MFRC522::PICC_CMD_MF_AUTH_KEY_A, trailerBlock, &key, &(mfrc522.uid)));
  status = (MFRC522::StatusCode) mfrc522.MIFARE_Read(blockAddr, buffer, &size);

  dump_byte_array(buffer, 16);

  mfrc522.PICC_HaltA();
  mfrc522.PCD_StopCrypto1();
}

//=========================================== RFID read

void dump_byte_array(byte *buffer, byte bufferSize) 
{
  for (byte i = 0; i < bufferSize; i++) 
    {
      addzero = (buffer[i] < 0x10 ? " 0" : " ");
      code = (buffer[i]);
      pass += addzero;
      pass += code;

      if (i ==  (bufferSize-1)) 
        {
        Serial.println(pass);
        pass = "";
        }
    }
}

//=========================================== Stepper reference

void ref() {
  ENDval = digitalRead(END);
  if (!ENDval) {
    while (!ENDval) {
      ENDval = digitalRead(END);
      stepper.setSpeed(50);
      stepper.runSpeed();
    }
    digitalWrite(sleep, HIGH);
    stepper.setCurrentPosition(0);
    RFID();
  }
  digitalWrite(sleep, HIGH);
}

//=========================================== Serial read

void serial() 
{
  if (Serial.available()>0)
   {
    data = Serial.readStringUntil('\n');
    Serial.println(data + "A");
   }
}
    
