# ========================================================================= Loading Libraries

from kivy.app import App
from kivy.properties import StringProperty, BooleanProperty, NumericProperty
from kivy.uix.screenmanager import ScreenManager, Screen
from kivy.core.window import Window
from kivy.clock import Clock
import serial

# ========================================================================= Setting up Serial port for Arduino connection

ser = serial.Serial('/dev/ttyACM0', 9600, timeout=1)  # if it doesent work change 0 to 1 or vice versa
ser.reset_input_buffer()

# ========================================================================= Declaration of global Variables

select = 1
coin = False
process = False
if str(open(f"temp/GIF", "r").read()).strip() == "True":
    GIF = True
else:
    GIF = False

# ========================================================================= Declaration of  class MainPage

class MainPage(Screen):

# ========================================================================= Class variables

    design_selected = StringProperty("designs/design_Austria.png")
    background_design = StringProperty("images/background_Austria.png")

# ========================================================================= Image slider function

    def show_img(self, direction):
        global select

# ========================================================================= Determine Image

        if direction == "minus":
            if select == 1:
                select = 6
            else:
                select -= 1
        if direction == "plus":
            if select == 6:
                select = 1
            else:
                select += 1

# ========================================================================= Switch Image to determined Image

        if select == 1:
            self.design_selected = "designs/design_Austria.png"
            self.background_design = "images/background_Austria.png"

        elif select == 2:
            self.design_selected = "designs/design_Germany.png"
            self.background_design = "images/background_Germany.png"

        elif select == 3:
            self.design_selected = "designs/design_Spain.png"
            self.background_design = "images/background_Spain.png"

        elif select == 4:
            self.design_selected = "designs/design_Norway.png"
            self.background_design = "images/background_Norway.png"

        elif select == 5:
            self.design_selected = "designs/design_Switzerland.png"
            self.background_design = "images/background_Switzerland.png"

        elif select == 6:
            self.design_selected = "designs/design_Luxenburg.png"
            self.background_design = "images/background_Luxenburg.png"

# ========================================================================= Declaration of class processing

class Processing(Screen):

# ========================================================================= Class variables

    i = 1
    Processing_gif = StringProperty("")

# ========================================================================= Process start function

    def cycle_start(self):
        global select
        global process

        process = True
# ========================================================================= Determining delay from files

        if select == 1:
            delay = (str(open(f"temp/Input1", "r").read())).strip()
        elif select == 2:
            delay = (str(open(f"temp/Input2", "r").read())).strip()
        elif select == 3:
            delay = (str(open(f"temp/Input3", "r").read())).strip()
        elif select == 4:
            delay = (str(open(f"temp/Input4", "r").read())).strip()
        elif select == 5:
            delay = (str(open(f"temp/Input5", "r").read())).strip()
        elif select == 6:
            delay = (str(open(f"temp/Input6", "r").read())).strip()

# ========================================================================= Writing determined design and delay on serial por
        data = ("Data-" + str(select) + "-" + str(delay) +  "\n").encode()
        if select > 0:
            ser.write(data)
        

# ========================================================================= Process end function

    @staticmethod
    def timeover(self):
        global coin
        global process

# ========================================================================= Switch Screen back and reset variables

        App.get_running_app().root.current = "wait"
        coin = False
        process = False

# ========================================================================= Processing gif function

    def gif_Processing(self, dt):

        global GIF

# ========================================================================= Determine if gif is enabled and the machine is producing
# ========================================================================= If true animate gif

        if GIF and App.get_running_app().root.current == "pro":

            if self.i <= 58:
                self.i += 1
            else:
                self.i = 0

            self.Processing_gif = "images/gif_Process/Process-" + str(self.i) + ".jpg"
        else:
            self.Processing_gif = "images/gif_Process/Process-0.jpg"

# ========================================================================= Setup for kivy clock

    def __init__(self, **kwargs):
        super(Processing, self).__init__()
        Clock.schedule_interval(self.gif_Processing, 0.02)
        Clock.schedule_once(self.gif_Processing, 5)

# ========================================================================= Declaration of class Waiting

class Waiting(Screen):

# ========================================================================= Class variables

    coin = False
    i = 1
    Waiting_gif = StringProperty("")

# ========================================================================= Coin check function | Switch Screen when coin is inserted

    def coin_check(self):

        if App.get_running_app().root.current == "wait":
            App.get_running_app().root.current = "main"

# ========================================================================= Determine if gif is enabled and the machine is waiting
# ========================================================================= If true animate gif

    def gif_Waiting(self, dt):

        global GIF

        if GIF and App.get_running_app().root.current == "wait":

            if self.i <= 6:
                self.i += 1
            else:
                self.i = 0

            self.Waiting_gif = "images/gif_Waiting/Coin_waiting-" + str(self.i) + ".jpg"
        else:
            self.Waiting_gif = "images/gif_Waiting/Coin_waiting-0.jpg"

# ========================================================================= Setup for kivy clock

    def __init__(self, **kwargs):
        super(Waiting, self).__init__()
        Clock.schedule_interval(self.gif_Waiting, 0.1)
        Clock.schedule_once(self.gif_Waiting, 5)

# ========================================================================= Declaration of class DevPage

class DevPage(Screen):

# ========================================================================= Class variables

    gif1 = 1
    gif2 = 2
    encoding = "01 02 03 04 05 06 07 08 09 010 255 011 012 013 014 015"
    data = ""
    dev = False
    

    Input1_enabled = BooleanProperty(False)
    Input2_enabled = BooleanProperty(False)
    Input3_enabled = BooleanProperty(False)
    Input4_enabled = BooleanProperty(False)
    Input5_enabled = BooleanProperty(False)
    Input6_enabled = BooleanProperty(False)
    pad_enabled = BooleanProperty(False)

    Input1_text = StringProperty(open(f"temp/Input1", "r").read())
    Input2_text = StringProperty(open(f"temp/Input2", "r").read())
    Input3_text = StringProperty(open(f"temp/Input3", "r").read())
    Input4_text = StringProperty(open(f"temp/Input4", "r").read())
    Input5_text = StringProperty(open(f"temp/Input5", "r").read())
    Input6_text = StringProperty(open(f"temp/Input6", "r").read())

    Label1_enabled = BooleanProperty(True)

    Label1_text = StringProperty("NA")
    
    toggle1_state = StringProperty("normal")

    
    Slider_M = NumericProperty(open(f"temp/Slider_SM", "r").read())
    Slider_S = NumericProperty(open(f"temp/Slider_SS", "r").read())
    Slider_T = NumericProperty(open(f"temp/Slider_TO", "r").read())

# ========================================================================= Read Serial bus and check what is received
# ========================================================================= Moreover call function or set variables

    def serial(self, dt):
        global coin

# ========================================================================= read the data from port

        if ser.inWaiting() > 0:
            self.data = ser.readline().decode('utf-8', errors="ignore").strip()
            self.data = str(self.data)
            print(self.data)

# ========================================================================= checks if data is the rfid pin

        if self.data == self.encoding:
            self.data = ""
            if self.dev == False:
                App.get_running_app().root.current = "dev"
                self.dev = True
            else:
                self.rfid_disabled()
                self.dev = False

# ========================================================================= checks data if a coin was inserted

        if self.data == "Coin inserted":
            self.data = ""
            coin = True
            Waiting().coin_check()

# ========================================================================= Checks if the stepper is finished

        if self.data == "Stepper fertig":
            Processing().timeover(self)
        data2 = self.data.split("-")


# ========================================================================= disables the DevPage
    
    def rfid_disabled(self):
        global coin
        if coin:
            if process:
                App.get_running_app().root.current = "pro"
            else:
                App.get_running_app().root.current = "main"
        else:
            App.get_running_app().root.current = "wait"

# ========================================================================= function that  changes the screen settings

    @staticmethod
    def fullscreen_enabled(widget):
        if widget.active:
            Window.fullscreen = "auto"
        else:
            Window.fullscreen = False

# ========================================================================= toggle function looks if a button was switched
# ========================================================================= therefrom enabling or disabling things

    def on_toggle_button_state(self, widget):

        global GIF
        

        if widget.state == "down":
            self.Label1_enabled = False
            self.Label1_text = "Disabled"
            GIF = False

            with open("temp/GIF", "w") as file:
                file.write(str(GIF))
            file.close()

        else:
            self.Label1_enabled = True 
            self.Label1_text = "Enabled"
            GIF = True

            with open("temp/GIF", "w") as file:
                file.write(str(GIF))
            file.close()

    def save_slider(self, slider):
        if slider == "TO":
            with open("temp/Slider_TO", "w") as file:
                file.write(str(int(self.ids.TO.value)))
            file.close()
            status = ("TO-" + str(int(self.ids.TO.value)) + "\n").encode()
            
        elif slider == "SM":
            with open("temp/Slider_SM", "w") as file:
                file.write(str(int(self.ids.SM.value)))
            file.close()
            status = ("SM-" + str(int(self.ids.SM.value)) + "\n").encode()

        elif slider == "SS":
            with open("temp/Slider_SS", "w") as file:
                file.write(str(int(self.ids.SS.value)))
            file.close()
            status = ("SS-" + str(int(self.ids.SS.value)) + "\n").encode()
        ser.write(status)

    
    def motor_turn_l(self):
        command = ("Motor-left" + "\n").encode()
        ser.write(command)
    
    def motor_turn_r(self):
        command = ("Motor-right" + "\n").encode()
        ser.write(command)
    
    def motor_stop(self):
        command = ("Motor-stop" + "\n").encode()
        ser.write(command)
        
    
# ========================================================================= function that writes numbers according to the pressed buttons into an input box

    def write(self, Number):
        if self.Input1_enabled and len(self.Input1_text) < 5:
            if self.Input1_text == "0":
                self.Input1_text = self.Input1_text[:-1]
            self.Input1_text += Number
        if self.Input2_enabled and len(self.Input2_text) < 5:
            if self.Input2_text == "0":
                self.Input2_text = self.Input2_text[:-1]
            self.Input2_text += Number
        if self.Input3_enabled and len(self.Input3_text) < 5:
            if self.Input3_text == "0":
                self.Input3_text = self.Input3_text[:-1]
            self.Input3_text += Number
        if self.Input4_enabled and len(self.Input4_text) < 5:
            if self.Input4_text == "0":
                self.Input4_text = self.Input4_text[:-1]
            self.Input4_text += Number
        if self.Input5_enabled and len(self.Input5_text) < 5:
            if self.Input5_text == "0":
                self.Input5_text = self.Input5_text[:-1]
            self.Input5_text += Number
        if self.Input6_enabled and len(self.Input6_text) < 5:
            if self.Input6_text == "0":
                self.Input6_text = self.Input6_text[:-1]
            self.Input6_text += Number

# ========================================================================= function that erases characters from input box

    def erase(self):
        if self.Input1_enabled and len(self.Input1_text) >= 1:
            self.Input1_text = self.Input1_text[:-1]
        if self.Input2_enabled and len(self.Input2_text) >= 1:
            self.Input2_text = self.Input2_text[:-1]
        if self.Input3_enabled and len(self.Input3_text) >= 1:
            self.Input3_text = self.Input3_text[:-1]
        if self.Input4_enabled and len(self.Input4_text) >= 1:
            self.Input4_text = self.Input4_text[:-1]
        if self.Input5_enabled and len(self.Input5_text) >= 1:
            self.Input5_text = self.Input5_text[:-1]
        if self.Input6_enabled and len(self.Input6_text) >= 1:
            self.Input6_text = self.Input6_text[:-1]

# ========================================================================= function that looks which input box is focused at the moment

    def focus(self, Input):
        self.prevent_range()

        if Input != "Input1_enabled":
            self.Input1_enabled = False
            with open("temp/Input1", "w") as file:
                file.write(self.Input1_text)
            file.close()
        else:
            self.Input1_enabled = True
        if Input != "Input2_enabled":
            self.Input2_enabled = False
            with open("temp/Input2", "w") as file:
                file.write(self.Input2_text)
            file.close()
        else:
            self.Input2_enabled = True
        if Input != "Input3_enabled":
            self.Input3_enabled = False
            with open("temp/Input3", "w") as file:
                file.write(self.Input3_text)
            file.close()
        else:
            self.Input3_enabled = True
        if Input != "Input4_enabled":
            self.Input4_enabled = False
            with open("temp/Input4", "w") as file:
                file.write(self.Input4_text)
            file.close()
        else:
            self.Input4_enabled = True
        if Input != "Input5_enabled":
            self.Input5_enabled = False
            with open("temp/Input5", "w") as file:
                file.write(self.Input5_text)
            file.close()
        else:
            self.Input5_enabled = True
        if Input != "Input6_enabled":
            self.Input6_enabled = False
            with open("temp/Input6", "w") as file:
                file.write(self.Input6_text)
            file.close()
        else:
            self.Input6_enabled = True

        if self.Input1_enabled and self.Input2_enabled and self.Input3_enabled and self.Input4_enabled and self.Input5_enabled and self.Input6_enabled == False:
            pass
        else:
            self.pad_enabled = True

# ========================================================================= function that saves the input and eliminates everything what is wrong

    def enter(self):

        self.prevent_range()
        self.erase_zeros()
        self.Input_save()

        self.Input1_enabled = False
        self.Input2_enabled = False
        self.Input3_enabled = False
        self.Input4_enabled = False
        self.Input5_enabled = False
        self.Input6_enabled = False

        if not self.Input1_enabled and not self.Input2_enabled and not self.Input3_enabled and not self.Input4_enabled and not self.Input5_enabled and not self.Input6_enabled:
            self.pad_enabled = False

# ========================================================================= function that prevents the input box to run out of range

    def prevent_range(self):
        if self.Input1_text == "":
            self.Input1_text = "0"
        if self.Input2_text == "":
            self.Input2_text = "0"
        if self.Input3_text == "":
            self.Input3_text = "0"
        if self.Input4_text == "":
            self.Input4_text = "0"
        if self.Input5_text == "":
            self.Input5_text = "0"
        if self.Input6_text == "":
            self.Input6_text = "0"

# ========================================================================= function that erases unsignificant zeros

    def erase_zeros(self):
        while self.Input1_text[0] == "0" and len(self.Input1_text) > 1:
            self.Input1_text = self.Input1_text[1:]
        while self.Input2_text[0] == "0" and len(self.Input2_text) > 1:
            self.Input2_text = self.Input2_text[1:]
        while self.Input3_text[0] == "0" and len(self.Input3_text) > 1:
            self.Input3_text = self.Input3_text[1:]
        while self.Input4_text[0] == "0" and len(self.Input4_text) > 1:
            self.Input4_text = self.Input4_text[1:]
        while self.Input5_text[0] == "0" and len(self.Input5_text) > 1:
            self.Input5_text = self.Input5_text[1:]
        while self.Input6_text[0] == "0" and len(self.Input6_text) > 1:
            self.Input6_text = self.Input6_text[1:]

# ========================================================================= function that saves the values of the input boxes into a file

    def Input_save(self):
        with open("temp/Input1", "w") as file:
            file.write(self.Input1_text)
        file.close()
        with open("temp/Input2", "w") as file:
            file.write(self.Input2_text)
        file.close()
        with open("temp/Input3", "w") as file:
            file.write(self.Input3_text)
        file.close()
        with open("temp/Input4", "w") as file:
            file.write(self.Input4_text)
        file.close()
        with open("temp/Input5", "w") as file:
            file.write(self.Input5_text)
        file.close()
        with open("temp/Input6", "w") as file:
            file.write(self.Input6_text)

# ========================================================================= function that subtracts or adds one to the input box value according to a button press

    def Input_buttons(self, direction, Input):
        if direction == "minus":
            if Input == "Input1" and int(self.Input1_text) > 0:
                self.Input1_text = str(int(self.Input1_text) - 1)
            elif Input == "Input2" and int(self.Input2_text) > 0:
                self.Input2_text = str(int(self.Input2_text) - 1)
            elif Input == "Input3" and int(self.Input3_text) > 0:
                self.Input3_text = str(int(self.Input3_text) - 1)
            elif Input == "Input4" and int(self.Input4_text) > 0:
                self.Input4_text = str(int(self.Input4_text) - 1)
            elif Input == "Input5" and int(self.Input5_text) > 0:
                self.Input5_text = str(int(self.Input5_text) - 1)
            elif Input == "Input6" and int(self.Input6_text) > 0:
                self.Input6_text = str(int(self.Input6_text) - 1)

            self.Input_save()

        elif direction == "plus":
            if Input == "Input1" and int(self.Input1_text) < 99999:
                self.Input1_text = str(int(self.Input1_text) + 1)
            if Input == "Input2" and int(self.Input2_text) < 99999:
                self.Input2_text = str(int(self.Input2_text) + 1)
            if Input == "Input3" and int(self.Input3_text) < 99999:
                self.Input3_text = str(int(self.Input3_text) + 1)
            if Input == "Input4" and int(self.Input4_text) < 99999:
                self.Input4_text = str(int(self.Input4_text) + 1)
            if Input == "Input5" and int(self.Input5_text) < 99999:
                self.Input5_text = str(int(self.Input5_text) + 1)
            if Input == "Input6" and int(self.Input6_text) < 99999:
                self.Input6_text = str(int(self.Input6_text) + 1)

            self.Input_save()

# ========================================================================= Initialization
    
    def slider_init(self, dt):
        self.save_slider("TO")
        self.save_slider("SM")
        self.save_slider("SS")
    
    def __init__(self, **kwargs):
        super(DevPage, self).__init__()

        Clock.schedule_interval(self.serial, 0.1)
        Clock.schedule_once(self.serial, 5)
        
        Clock.schedule_once(self.slider_init, 1)
        
        if GIF:
            self.toggle1_state = "normal"
        else:
            self.toggle1_state = "down"
        
        self.on_toggle_button_state(self.ids.toggle_1)
        
# ========================================================================= Manager of the different pages

class WindowManager(ScreenManager):
    Window.fullscreen = "auto"

# ========================================================================= Interface builder

class Interface(App):
    def build(self):
        pass


if __name__ == "__main__":
    Interface().run()
